# -*- coding: utf-8 -*-
"""THIS ENTIRE MODULE/FOLDER IS DEPRECATED!
IT HAS TO BE REMOVED IN THE NEXT VERSION OF THIS ADD-ON.
IT'S HERE JUST TO PREVENT UNWANTED ERRORS WHILE UPGRADING FROM THE PREVIOUS
VERSION!
TODO
"""
import logging
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('collective.smartlink')
logger = logging.getLogger(__name__)



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import collective.smartlink


class CollectiveSmartlinkLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML(package=collective.smartlink)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.smartlink:default')


COLLECTIVE_SMARTLINK_FIXTURE = CollectiveSmartlinkLayer()


COLLECTIVE_SMARTLINK_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_SMARTLINK_FIXTURE,),
    name='CollectiveSmartlinkLayer:IntegrationTesting'
)


COLLECTIVE_SMARTLINK_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_SMARTLINK_FIXTURE,),
    name='CollectiveSmartlinkLayer:FunctionalTesting'
)


COLLECTIVE_SMARTLINK_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_SMARTLINK_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='CollectiveSmartlinkLayer:AcceptanceTesting'
)



# -*- coding: utf-8 -*-
from collective.smartlink import _
from plone.app.contenttypes.interfaces import ILink
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


class ISmartLinkExtension(Interface):
    """ Marker interface for Links.
    """



# -*- coding: utf-8 -*-

from plone.dexterity.interfaces import IDexterityFTI
from zope.component import queryUtility
from remove_internal_link import fix_internal_link_field

import logging

logger = logging.getLogger(__name__)


default_profile = 'profile-collective.smartlink:default'


def to_1100(context):
    """
    """
    logger.info('Upgrading collective.smartlink to version 1100')
    fti = queryUtility(IDexterityFTI, name="Link", default=None)
    if not fti:
        return

    logger.info('Changing model_file for Link type.')
    fti._updateProperty('model_file', 'plone.app.contenttypes.schema:link.xml')

    fix_internal_link_field(context)



# -*- coding: utf-8 -*-
from plone.app.contenttypes.browser.link_redirect_view import LinkRedirectView as Base  # noqa
from plone.app.contenttypes.browser.link_redirect_view import NON_RESOLVABLE_URL_SCHEMES  # noqa
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api


class LinkRedirectView(Base):
    index = ViewPageTemplateFile('templates/link.pt')

    def absolute_target_url(self):
        """Compute the absolute target URL."""
        url = self.url()

        if self.context.remoteUrl.startswith('$'):
            # searching for the linked object
            obj = api.content.get(UID=url.split('/')[-1])
            return obj.absolute_url()
        else:
            return super(LinkRedirectView, self).absolute_target_url()



# -*- coding: utf-8 -*-
from App.config import getConfiguration
from collective.smartlink.interfaces import ICollectiveSmartlinkLayer
from lxml import etree
from lxml.html import fromstring
from os import environ
from plone.app.theming.interfaces import IThemingLayer
from plone.app.theming.utils import compileThemeTransform
from plone.app.theming.utils import findContext
from plone.app.theming.utils import getParser
from plone.app.theming.utils import prepareThemeParameters
from plone.app.theming.utils import theming_policy
from plone.app.theming.zmi import patch_zmi
from plone.app.uuid.utils import uuidToURL
from plone.transformchain.interfaces import ITransform
from repoze.xmliter.serializer import XMLSerializer
from repoze.xmliter.utils import getHTMLSerializer
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


import logging


LOGGER = logging.getLogger('collective.smartlink.transform')
LOGGER.setLevel(logging.WARNING)


@implementer(ITransform)
@adapter(Interface, ICollectiveSmartlinkLayer)
class UidLinkTransform(object):
    """Late stage in the 8000's transform chain.
    When plone.app.blocks is used, we can benefit from lxml parsing having
    taken place already.
    """

    order = 8860

    def __init__(self, published, request):
        self.published = published
        self.request = request

    def debug_theme(self):
        ''' Check if the theme should be debugged
        We will debug the theme
        when we have a truish diazo.debug parameter in the request
        '''
        if not getConfiguration().debug_mode:
            return False
        diazo_debug = self.request.get('diazo.debug', '').lower()
        return diazo_debug in ('1', 'y', 'yes', 't', 'true')

    def develop_theme(self):
        ''' Check if the theme should be recompiled
        every time the transform is applied
        '''
        if not getConfiguration().debug_mode:
            return False
        if self.debug_theme():
            return True
        if environ.get('DIAZO_ALWAYS_CACHE_RULES'):
            return False
        return True

    def setupTransform(self, runtrace=False):
        debug_mode = self.develop_theme()
        policy = theming_policy(self.request)

        # Obtain settings. Do nothing if not found
        settings = policy.getSettings()

        if settings is None:
            return None

        if not policy.isThemeEnabled():
            return None

        cache = policy.getCache()

        # Apply theme
        transform = None

        if not debug_mode:
            transform = cache.transform

        if transform is None:
            rules = settings.rules
            absolutePrefix = settings.absolutePrefix or None
            readNetwork = settings.readNetwork
            parameterExpressions = settings.parameterExpressions

            transform = compileThemeTransform(
                rules,
                absolutePrefix,
                readNetwork,
                parameterExpressions,
                runtrace=runtrace
            )
            if transform is None:
                return None

            if not debug_mode:
                cache.updateTransform(transform)

        return transform

    def getSettings(self):
        return theming_policy(self.request).getSettings()

    def parseTree(self, result):
        contentType = self.request.response.getHeader('Content-Type')
        if contentType is None or not contentType.startswith('text/html'):
            return None

        contentEncoding = self.request.response.getHeader('Content-Encoding')
        if contentEncoding \
           and contentEncoding in ('zip', 'deflate', 'compress',):
            return None

        try:
            return getHTMLSerializer(result, pretty_print=False)
        except (AttributeError, TypeError, etree.ParseError):
            return None

    def transformBytes(self, result, encoding):
        try:
            result = result.decode(encoding)
        except UnicodeDecodeError:
            # This is probably a file or an image
            # FIXME probably we do not event want to apply
            # this transform for files and images
            pass
        return self.transformIterable([result], encoding)

    def transformString(self, result, encoding):
        return self.transformIterable([result], encoding)

    def transformUnicode(self, result, encoding):
        return self.transformIterable([result], encoding)

    def transformIterable(self, result, encoding):
        """Apply the transform if required
        """
        if isinstance(result, XMLSerializer):
            root = result.tree.getroot()
            for (element, attribute, link, pos) in root.iterlinks():
                if 'resolveuid' in link:
                    uuid_value = link.split('/')[-1]
                    url = uuidToURL(uuid_value)
                    LOGGER.info("\n---")
                    LOGGER.info("link found in page: {}".format(link))
                    LOGGER.info(
                        "link extracted uuid_value: {}".format(uuid_value)
                    )
                    LOGGER.info("url: {}".format(url))
                    LOGGER.info("\n---")
                    if url:
                        element.set(attribute, url)

        return result



# -*- coding: utf-8 -*-

from plone import api
from plone.api.content import get_uuid

import logging
import transaction

logger = logging.getLogger(__name__)


def fix_internal_link_field(context):
    """ Parte della migrazione collective.smartlink to version 1100.
    Serve per togliere il contenuto del campo 'internal_link' e metterlo
    nel campo standard 'remoteUrl'.

    Si spazzola tutti i link del sito e li corregge.
    """
    logger.info('Fixing the internal_link field for all the Link objects.')

    site = api.portal.get()
    print u"Getting all the Link objects in the site..."
    link_brains = site.portal_catalog.unrestrictedSearchResults(
        portal_type=["Link"],
    )

    objects_changed = 0
    total_changed = 0
    logger.info("Found {} Link objects".format(len(link_brains)))

    for brain in link_brains:
        link_obj = brain.getObject()
        # Facciamo un controllo che dovrebbe essere inutile ma stiamo dalla
        # parte dei bottoni. Se un oggetto ha un link interno NON può avere
        # anche un link esterno.
        if link_obj.internal_link:
            logger.info('---Link: {} | punta a: {}\n'.format(
                link_obj.absolute_url_path(),
                link_obj.internal_link.to_path,
            ))

            linked_obj = link_obj.internal_link.to_object
            uuid = get_uuid(linked_obj)
            link_obj.remoteUrl = u'${}/resolveuid/{}'.format(
                '{portal_url}',
                uuid,
            )
            link_obj.internal_link = None
            link_obj.reindexObject(idxs=["remoteUrl"])

            objects_changed += 1
            total_changed += 1
            if objects_changed > 10:
                try:
                    print "Partial Commit..."
                    transaction.commit()
                    print "Partial Commit: OK"
                except Exception as e:
                    logger.error(
                        u"Error while committing transaction.")
                    logger.error(u"{}".format(e))
                objects_changed = 0
    logger.info("Changed a total of {} Link objects".format(total_changed))
    transaction.commit()
